@@ -34,7 +34,7 @@
 
 #include "rb-gst-media-types.h"
 
-#define SOURCE_ENCODING_TARGET_FILE "../data/rhythmbox.gep"
+#define SOURCE_ENCODING_TARGET_FILE "data/rhythmbox.gep"
 #define INSTALLED_ENCODING_TARGET_FILE DATADIR"/sound-juicer/rhythmbox.gep"
 static GstEncodingTarget *default_target = NULL;
 
