@@ -118,9 +118,9 @@ static CellCbContext cell_editing_context;
 
 #define DEFAULT_PARANOIA 15
 #define RAISE_WINDOW "raise-window"
-#define SOURCE_BUILDER "../data/sound-juicer.ui"
+#define SOURCE_BUILDER "data/sound-juicer.ui"
 #define INSTALLED_BUILDER DATADIR"/sound-juicer/sound-juicer.ui"
-#define SOURCE_MENU_BUILDER "../data/sound-juicer-menu.ui"
+#define SOURCE_MENU_BUILDER "data/sound-juicer-menu.ui"
 #define INSTALLED_MENU_BUILDER DATADIR"/sound-juicer/sound-juicer-menu.ui"
 #define COMPOSER_ROW 2 /* Row of entry_table containing composer_entry */
 
@@ -1117,18 +1117,12 @@ static void audio_volume_changed_cb (GSettings *settings, gchar *key, gpointer u
 static void
 metadata_cb (SjMetadataGetter *m, GList *albums, GError *error)
 {
-  gboolean realized = gtk_widget_get_realized (main_window);
-
   set_action_state ("re-read(false)");
-
-  if (realized)
-    gdk_window_set_cursor (gtk_widget_get_window (main_window), NULL);
-    /* Clear the statusbar message */
-    gtk_statusbar_pop(GTK_STATUSBAR(status_bar), 0);
-
   if (error && !(error->code == SJ_ERROR_CD_NO_MEDIA)) {
+    gboolean realized;
     GtkWidget *dialog;
 
+    realized = gtk_widget_get_realized (main_window);
     dialog = gtk_message_dialog_new_with_markup (realized ? GTK_WINDOW (main_window) : NULL, 0,
                                                  GTK_MESSAGE_ERROR,
                                                  GTK_BUTTONS_CLOSE,
@@ -1208,11 +1202,6 @@ static void reread_cd (gboolean ignore_no_media)
 {
   /* TODO: remove ignore_no_media? */
   GError *error = NULL;
-  GdkCursor *cursor;
-  GdkWindow *window;
-  gboolean realized = gtk_widget_get_realized (main_window);
-
-  window = gtk_widget_get_window (main_window);
 
   set_action_state ("re-read(true)");
   set_action_enabled ("re-read", FALSE);
@@ -1220,17 +1209,6 @@ static void reread_cd (gboolean ignore_no_media)
   /* Make sure nothing is playing */
   stop_playback ();
 
-  /* Set watch cursor */
-  if (realized) {
-    cursor = gdk_cursor_new_for_display (gtk_widget_get_display (GTK_WIDGET (main_window)), GDK_WATCH);
-    gdk_window_set_cursor (window, cursor);
-    g_object_unref (cursor);
-    gdk_display_sync (gtk_widget_get_display (GTK_WIDGET (main_window)));
-  }
-
-  /* Set statusbar message */
-  gtk_statusbar_push(GTK_STATUSBAR(status_bar), 0, _("Retrieving track listing...please wait."));
-
   if (!drive)
     sj_debug (DEBUG_CD, "Attempting to re-read NULL drive\n");
 
@@ -1242,17 +1220,16 @@ static void reread_cd (gboolean ignore_no_media)
     sj_debug (DEBUG_CD, "Media is not an audio CD\n");
     set_action_state ("re-read(false)");
     update_ui_for_album (NULL);
-    gtk_statusbar_pop(GTK_STATUSBAR(status_bar), 0);
-    if (realized)
-      gdk_window_set_cursor (window, NULL);
     return;
   }
 
   sj_metadata_getter_list_albums (metadata, &error);
 
   if (error && !(error->code == SJ_ERROR_CD_NO_MEDIA && ignore_no_media)) {
+    gboolean realized;
     GtkWidget *dialog;
 
+    realized = gtk_widget_get_realized (main_window);
     dialog = gtk_message_dialog_new (realized ? GTK_WINDOW (main_window) : NULL, 0,
                                      GTK_MESSAGE_ERROR,
                                      GTK_BUTTONS_CLOSE,
@@ -2030,6 +2007,34 @@ is_cd_duplication_available(void)
   return FALSE;
 }
 
+static void
+ui_set_retrieving_metadata (gboolean state)
+{
+  if (state) {
+    gtk_statusbar_push(GTK_STATUSBAR(status_bar), 1,
+                       _("Retrieving track listing...please wait."));
+    g_application_mark_busy (g_application_get_default ());
+  } else {
+    gtk_statusbar_pop(GTK_STATUSBAR(status_bar), 1);
+    g_application_unmark_busy (g_application_get_default ());
+  }
+}
+
+static void
+reread_state_changed_cb (gchar    *group_name,
+                         gchar    *action_name,
+                         GVariant *value,
+                         gpointer  user_data)
+{
+  static gboolean state = FALSE;
+
+  if (g_variant_get_boolean (value) == state)
+    return;
+
+  state = !state;
+  ui_set_retrieving_metadata (state);
+}
+
 GActionEntry app_entries[] = {
   { "re-read", on_reread_activate, NULL, "false", NULL },
   { "duplicate", on_duplicate_activate, NULL, NULL, NULL },
@@ -2108,6 +2113,10 @@ startup_cb (GApplication *app, gpointer user_data)
   g_action_map_add_action_entries (G_ACTION_MAP (app),
                                    app_entries, G_N_ELEMENTS (app_entries),
                                    NULL);
+  g_signal_connect (app,
+                    "action-state-changed::re-read",
+                    G_CALLBACK (reread_state_changed_cb),
+                    NULL);
 
   builder = gtk_builder_new ();
   if (g_file_test (SOURCE_BUILDER, G_FILE_TEST_EXISTS) != FALSE) {
@@ -2472,6 +2481,7 @@ int main (int argc, char **argv)
   app = gtk_application_new ("org.gnome.sound-juicer",
                              G_APPLICATION_FLAGS_NONE);
 
+  g_object_set (app, "register-session", TRUE, NULL);
   g_signal_connect (app, "startup", G_CALLBACK (startup_cb), NULL);
   g_signal_connect (app, "activate", G_CALLBACK (activate_cb), NULL);
 
